package cz.muni.pa165.rating.config;

import cz.muni.pa165.core.exception.ResourceNotFoundException;
import cz.muni.pa165.rating.MockedEntities;
import cz.muni.pa165.rating.data.model.RatingFilter;
import cz.muni.pa165.rating.data.repository.MovieRatingRepository;
import cz.muni.pa165.rating.service.MovieRatingService;
import jakarta.annotation.PostConstruct;
import java.util.Optional;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.springframework.boot.test.context.TestConfiguration;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Primary;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;

@TestConfiguration
public class MockConfiguration {

    @Mock
    private MovieRatingRepository repository;

    @PostConstruct
    public void setup() {
        MockitoAnnotations.openMocks(this);
        mockRepositoryMethods();
    }

    @Bean
    @Primary
    public MovieRatingService movieRatingService() {
        return new MovieRatingService(repository);
    }

    private void mockRepositoryMethods() {
        Mockito
            .doThrow(ResourceNotFoundException.class)
            .when(repository)
            .findById(Mockito.anyLong());
        Mockito
            .doReturn(Optional.of(MockedEntities.RATING_1))
            .when(repository)
            .findById(MockedEntities.RATING_1.getId());
        Mockito
            .doReturn(new PageImpl<>(MockedEntities.RATINGS))
            .when(repository)
            .findAll(Mockito.any(Pageable.class));
        Mockito
            .doReturn(new PageImpl<>(MockedEntities.RATINGS))
            .when(repository)
            .findAllFiltered(
                Mockito.any(RatingFilter.class),
                Mockito.any(Pageable.class)
            );
        Mockito
            .doReturn(MockedEntities.RATING_1)
            .when(repository)
            .save(MockedEntities.RATING_1);
    }
}
